#!/usr/bin/perl

#This program reads GFDL output data obtained through wgrib and converts it to HSA (lf) format.
#Isha M. Renta Lopez
#slightly modified by Nicholas Carrasco (NOAA/AOML/HRD;UMiami/RSMAS/CIMAS); 
#- changes made to accept filename base at runtime, and to dynamically retrieve date and time from a spec file

$filenamebase = $ARGV[0];

#open the data files
open (GFDLU,'<',$filenamebase.".UGRD.txt") || die "Error opening file.";
open (GFDLV,'<',$filenamebase.".VGRD.txt") || die "Error opening file.";
open (GFDLP,'<',$filenamebase.".PRMSL.txt") || die "Error opening file.";
open (GFDLT,'<',$filenamebase.".TMPsfc.txt") || die "Error opening file.";
open (GFDLR,'<',$filenamebase.".RH.txt") || die "Error opening file.";
open (GFDLH,'<',$filenamebase.".HGT.txt") || die "Error opening file.";
open (SPEC,'<',$filenamebase.".spec.txt") || die "Error opening file.";

#creates lat/lon files
open (LAT,'>',$filenamebase.".lat.txt") || die "Error opening file.";
open (LON,'>',$filenamebase.".lon.txt") || die "Error opening file.";

#assign the data files to a list
@lgfdlp=<GFDLP>;
@lgfdlu=<GFDLU>;
@lgfdlv=<GFDLV>;
@lgfdlt=<GFDLT>;
@lgfdlr=<GFDLR>;
@lgfdlh=<GFDLH>;

#removes the first line from the list
splice(@lgfdlp,0,1,());
splice(@lgfdlu,0,1,());
splice(@lgfdlv,0,1,());
splice(@lgfdlt,0,1,());
splice(@lgfdlr,0,1,());
splice(@lgfdlh,0,1,());


open (U10M,'>>',$filenamebase.".U10M.txt") || die "Error opening file.";
open (V10M,'>>',$filenamebase.".V10M.txt") || die "Error opening file.";

#creates lists
@z0=();
@W35m=();
@Ust=();
$kconst=0.4;
@W10=();
@theta10=();
@U10=();
@V10=();

#converts the 35m winds into 10m winds
for($d=0;$d<3600;$d++)
   {
    $W35m[$d]=sqrt($lgfdlu[$d]**2+$lgfdlv[$d]**2); #magnitude of 35m winds
    
    if($W35m[$d]<=12.5)
        {
	#35 to 10m wind reduction equation for W35 less than 12.5 m/s
        $z0[$d]=1000*0.0185/9.8*(0.00053*$W35m[$d]**2+0.0286*$W35m[$d]-0.0137)**2; 
        }
    else
        {
	#35 to 10m wind reduction equation for W5 greater than 12.5 m/s
	$z0[$d]=0.0739*$W35m[$d]-0.58;     
	}
  
    $z0[$d]=$z0[$d]*0.001; #changes z0 from mm to m
    $Ust[$d]=($W35m[$d]*$kconst)/(log(35/$z0[$d])); #Calculates U* - Powell et al. 2003 eq. (1)
    $W10[$d]=($Ust[$d]/$kconst)*log(10/$z0[$d]); #Calculates 10 m wind magnitude

#print "u:".$lgfdlu[$d]." v:".$lgfdlv[$d]."\n";
    $theta10[$d]=atan2($lgfdlu[$d],$lgfdlv[$d]); #calculates 35m wind direction. Assumed wdir35=wdir10, kept in radians
#print $theta10[$d]."\n";

    $U10[$d]=$W10[$d]*sin($theta10[$d]); #Calculates the U10 component
    $V10[$d]=$W10[$d]*cos($theta10[$d]); #Calculates the V10 component
    printf U10M "$U10[$d]\n"; 
    printf V10M "$V10[$d]\n";
    }

use POSIX
#read spec file and set needed values:
$l = 0;
while( $line = <SPEC> ) {
 if ( $l == 0 ) {
  @dd = split(/:/,$line);
  @dat = split(/ /,$dd[2]);
  $date = $dat[1]/100;
  $time = ($dat[1]%100)*10000;
  $fcast = substr($dat[10],0,- 2);

  $datetime = POSIX::mktime($time%100,($time/100)%100,$time/10000,$date%100,(($date/100)%100)-1,($date/10000)-1900,0,0,1);
  $fcasttime = $datetime += $fcast*3600;
  $fdate = POSIX::strftime("%y%m%d",POSIX::localtime($fcasttime));
  $ftime = POSIX::strftime("%H%M%S",POSIX::localtime($fcasttime));
 }

 if ($l == 4) {
  @dd = split(/ +/,$line);
  $ybeg = $dd[3];
 }
 if ($l == 5) {
  @dd = split(/ +/,$line);
  $xbeg = -1 * $dd[2];
 }

 $l+=1;
}
                                                                                                                                    
#calculates the lat/long and place it in a file

#$xbeg=82.125;
#$ybeg=15.708;

for($j=0;$j<60;$j++)
     {    
        for($k=0;$k<60;$k++)    
            {
            $lon=($xbeg-($k*(0.08333333)));
            $lat=($ybeg+($j*(0.08333333)));
            printf LAT "$lat\n";
            printf LON "$lon\n";
            }
     }

open (LATI,'<',$filenamebase.".lat.txt") || die "Error opening file.";
open (LONI,'<',$filenamebase.".lon.txt") || die "Error opening file.";
open (U10M2,'<',$filenamebase.".U10M.txt") || die "Error opening file.";
open (V10M2,'<',$filenamebase.".V10M.txt") || die "Error opening file.";
open (ALLF,'>',$filenamebase.".ALLF.hsa") || die "Error opening file.";

#assigns lat/lon and U10 and V10 files to lists
@lgfdllat=<LATI>;
@lgfdllon=<LONI>;
@U10m2=<U10M2>;
@V10m2=<V10M2>;

#organizes the data files in one file .ALLF.txt
for($i=0;$i<3600;$i++)    
    {
#        printf ALLF "04 040912. 000000";
        printf ALLF ("04 %.6d. %.6d",$fdate,$ftime);
        $fcontentlat=$lgfdllat[$i];
        printf ALLF (" %7.3f",$fcontentlat);
        $fcontentlon=$lgfdllon[$i];
        printf ALLF ("%8.3f",$fcontentlon);
        $pressv=$lgfdlp[$i]/100;
        printf ALLF (" %6.1f",$pressv);
        $fcontentt=$lgfdlt[$i];
        printf ALLF (" %6.1f",$fcontentt);
        $fcontentr=$lgfdlr[$i];
        printf ALLF (" %6.1f",$fcontentr);
        $fcontenth=$lgfdlh[$i];
        printf ALLF ("  %6.1f",$fcontenth);
        $U10m=$U10m2[$i];
        printf ALLF ("%6.1f",$U10m);
        $V10m=$V10m2[$i];
        printf ALLF (" %6.1f",$V10m);
        printf ALLF (" G%.3d\n",$fcast);
    }
close(GFDLU);
close(GFDLV);
close(GFDLH);
close(GFDLR);
close(GFDLT);
close(GFDLP);
close(LATI);
close(LONI);
close(U10M);
close(V10M);
close(ALLF);
